/* Sachov centrum - Knihovna zahjen Petr Kuera 2000*/
/** Deklarace typ datovch struktur pouitch k reprezentaci
    knihovny zahjen v pamti.
  */

#ifndef __DATASTR_H__
#define __DATASTR_H__ 1

#include "cotyvar.h"

/** HLOUBKA_ADRESARE - uruje hloubku jedn rovn adrese. */
#define HLOUBKA_ADRESARE 4
/** POCET_ADRES_V_BLOKU - uruje poet adres v jednom bloku adrese. */
#define POCET_ADRES_V_BLOKU 16 /*2^4*/
/*Konstanta urujc masku pro odmaskovn prvnch HLOUBKA_ADRESARE bit
 */
#define ZAKLADNI_MASKA 15

/*uruje, jestli maj bt pozice v souboru pakovny, nebo m bt ukldn
 pmo datov typ TPozice*/
#define PAKUJ_POZICE 1

/** tah_t - popis tahu s ECO a vhou vbru, pokud je vha men ne 0,
    jde o dosud nespecifikovanou vhu.
*/
typedef struct _tah {
  /*vha vbru tahu*/
  char vaha;
  /*psmeno ze zpisu ECO pozice vznikl tmto tahem*/
  char ECO_pismeno;
  /*slo ze zpisu ECO pozice vznikl tmto tahem*/
  char ECO_cislo;
  /*dvoubajtov popis tahu*/
  u16 tah;
  /*poad partie ve zdrojovm souboru knihovny, v n byl tah naposledy
    pidvn.*/
  int cis_partie;
  /*Souet vah ze vech parti, v nich byl tah pidvn*/
  int soucet_vah;
  /*1 pokud byl tah v posledn partii pidvn a jeho vha nebyla explicitn
    urena.*/
  int neurcena;
} tah_t;

/*Poten velikost pole tahy ve struktue pozice_t*/
#define POZICE_TAHY_POCATECNI_VELIKOST 10
/*Prstek velikosti pole tahy ve struktue pozice_t, pi jejm pekroen*/
#define POZICE_TAHY_DELTA 10

/*Konstanta urujc, e ECO pozice nen implicitn A00*/
#define NENI_IMPL_A00 0
/*Konstanta urujc, e ECO pozice je A00 implicitn v rmci knihovny*/
#define KNIH_IMPL_A00 1
/*Konstanta urujc, e ECO pozice je A00 implicitn pouze v rmci dan partie*/
#define PART_IMPL_A00 2

/** pozice_t - struktura popisujc pozici s tahy, kter z n vychz*/
typedef struct _pozice {
  /*Odkaz na dal pozici v seznamu pozic*/
  struct _pozice *dalsi_pozice;
  /*Haovac kl pozice popsan touto strukturou*/
  u32 hasovaci_klic;
  /*psmeno ze zpisu ECO popisovan pozice*/
  char ECO_pismeno;
  /*slo ze zpisu ECO popisovan pozice*/
  char ECO_cislo;
  /*Uruje, jestli ECO pozice je implicitn A00,
    je-li NENI_IMPL_A00, pak nejde o implicitn A00,
    je-li ECO_A00&KNIH_IMPL_A00, pak jde o implicitn A00 v rmci cel knihovny,
    je-li ECO_A00&PART_IMPL_A00, pak se jedn o A00 implicitn pouze v rmci dan partie
  */
  char ECO_A00;
  /*popisovan pozice*/
  TPozice pozice;
  /*poet tah vedoucch z tto pozice*/
  int pocet_tahu;
  /*souet vah urench tah v aktuln partii*/
  int soucet_vah;
  /*poet tah s neurenou vahou v souasn partii*/
  int pocet_neurcenych;
  /*poet tah s nenulovou vahou v souasn partii*/
  int pocet_nenulovych;
  /*poet parti, v nich se pozice vyskytla*/
  int pocet_partii;
  /*velikost pole tahy*/
  int vel_tahu;
  /*pole obsahujc tahy vedouc z tto pozice*/
  tah_t *tahy;
  /*vypoten pozice v souboru*/
  long poloha;
  /*slo partie, v n byl pidn posledn tah s nenulovou nebo neurenou vahou,
    nebo -1, pokud tam takov tah nen*/
  int part_nenul_tah;
} pozice_t;

/** blok_pozice_t - struktura popisujc obsah bloku s pozicemi, kter
    maj stejn haovac kl.
 */
typedef struct _blok_pozic {
  /*Poet pozic je jsou uloen v tto struktue*/
  int pocet_pozic;
  /*Haovac kl spolen vem pozicm uloenm v tto struktue*/
  u32 hasovaci_klic;
  /*Poloha bloku v souboru s knihovnou (vypoteno tsn ped uloenm)*/
  long poloha;
  /*Poet byt, kter blok v souboru bude zabrat*/
  long delka_bloku;
  /*Ukazatel na hlavu spojovho seznamu s pozicemi, kter jsou v tomto bloku uloeny.*/
  pozice_t *pozice;
} blok_pozic_t;

/** blok_adresare_t - struktura popisujc obsah bloku s adresem v pamti.*/
typedef struct _blok_adresare {
  /*Poloha bloku v souboru s knihovnou zahjen (vypoteno tsn ped uloenm).*/
  long poloha;
  /*
    rove bloku ve struktue popisujc obsah knihovny, uruje, kter skupina
    bit v haovacm kli se pouije pi pechodu na dal rove.
  */
  int uroven;
  /*
    Ukazatel na bloky pozic, kter jsou ureny hodnotou haovacho kle.
    Haovac kl je uloen implicitn pomoc indexu v poli, jde o 16 prvkov pole,
    na kad rovni se tedy tp podle tveice bit, pokud poskldme pslun
    tveice za sebou, dostaneme prefix kle dlouh 4*(uroven+1) bit.
    Pokud je prvek pole NULL, znamen to bu, e pozice s klem zanajcm
    danm prefixem se dosud nevyskytla, nebo e je vce pouitch kl s danm
    prefixem, v tom ppad mus bt pslun prvek pole adresy_adresaru ne NULL.
    Pokud je prvek ne NULL, je jedin kl s danm prefixem, v tom ppad mus
    bt prvek v poli adresy_adresaru se stejnm indexem NULL.
   */
  blok_pozic_t *adresy_pozic[POCET_ADRES_V_BLOKU];
  /*
    Ukazatele na bloky adres dalch rovn v hierarchick struktue popisujc
    knihovnu zahjen.
   */
  struct _blok_adresare *adresy_adresaru[POCET_ADRES_V_BLOKU];
} blok_adresare_t;


/*Nsledujc typy slou k popisu stavu souboru.*/

/*typ popisujc tah uloen do souboru*/
typedef struct _f_tah {
  /*Pravdpodobnost, s jakou m bt tah vybrn*/
  char vaha;
  /*Psmeno z kdu ECO tahu*/
  char ECO_pismeno;
  /*slo z kdu ECO tahu*/
  char ECO_cislo;
  /*Dvoubajtov popis tahu*/
  u16 tah;
} f_tah_t;

/*Struktura popisujc pozici uloenou v souboru s knohovnou zahjen.*/
typedef struct {
  /*
    Dal pozice ve spojovm seznamu pozic z jednoho bloku.
    Skuten adresa dal pozice je ZAC_ADR+dalsi_pozice.
   */
  long dalsi_pozice;
  /*
    Stav hry popisovan touto pozic.
   */
  TPozice pozice;
  /*char ECO_pismeno;*/
  /*char ECO_cislo;*/
  /*Poet tah, kter vedou z tto pozice.*/
  int pocet_tahu;
  /*Pole s tahy vedoucmi z tto pozice.*/
  f_tah_t *tahy;
} f_pozice_t;

/*Struktura popisujc blok pozic uloen v souboru.*/
typedef struct _f_blok_pozic {
  /*Dlka bloku v bytech (=sizeof(f_blok_pozic_t)+pocet_pozic*sizeof(f_pozice_t).*/
  long delka_bloku;
  /*Poet pozic uloench v tomto bloku.*/
  int pocet_pozic;
  /*Haovac kl spolen vem pozicm v bloku.*/
  u32 hasovaci_klic;
  /*
    Pozice prvn pozice bloku v souboru - posun od zatku pozic.
    Skuten adresa v souboru je ZAC_POZIC+adr_pozic.
   */
  long adr_pozic;
} f_blok_pozic_t;

/*Struktura popisujc blok adrese uloen v souboru.*/
typedef struct _f_blok_adresare {
  /*Poloha pole adresy v souboru. Skuten adresa zatku pole je ZAC_POZIC+zac_adres.*/
  long zac_adres;
  /*rove bloku v hierarchick struktue adrese.*/
  int uroven;
  /*
    Pole s adresami nsledujcch blok. Pokud je pslun prvek 0, nensleduje
    v jeho smru dn dal blok. Pokud m bitov zpis hodnotu 0<adresa>,
    jde o adresu bloku adres, pokud m tvar 1<adresa>, pak jde o adresu bloku
    pozic. Vechny adresy jsou relativn vzhledem k ZAC_ADR. Adresa 0 je tedy adresou
    zkladnho bloku, na nj se vak neme vyskytnout odkaz v dnm bloku adres.
   */
  long adresy[POCET_ADRES_V_BLOKU];
} f_blok_adresare_t;

#if PAKUJ_POZICE

/*Struktura popisujc typ TPozice (viz cotyvar.h) v souboru, je-li tento typ pakovn*/
typedef struct {
  /*Stav rody - jako v TPozice*/
  u8 roch;
  /*Stav pole mimochodem - jako v TPozice*/
  u8 mimoch;
  /*Je-li bl na tahu - jako v TPozice*/
  bool bily;
  /*
    Popis achovnice - achovnice 8*8 po dcch - byte popisuje dv nsledujc pole.
    Obsah pole je zapsn jako 6+hodnota z TPozice.sch. 
  */
  char sch[32];
} f_tpozice_t;

/*
  Pevede pozici ze struktury typu TPozice do struktury typu f_tpozice_t.

  parametr pozice vstupn pozice
  parametr fpozice vstupn pozice

  vrac 0 pokud nedojde k chyb, jinak kd chyby dan konstantami v souboru
          obkc.h
 */
int pakuj(TPozice *pozice, f_tpozice_t *fpozice);

/*
  Pevede pozici ze struktury typu f_tpozice_t do struktury typu TPozice.

  parametr fpozice vstupn pozice
  parametr pozice vstupn pozice

  vrac 0 pokud nedojde k chyb, jinak kd chyby dan konstantami v souboru
          obkc.h
 */
int rozpakuj(f_tpozice_t *fpozice, TPozice *pozice);

#endif

/*Zatek blok adrese v souboru - poloha zkladnho bloku.*/
extern long ZAC_ADR;
/*Zatek blok s pozicemi v souboru.*/
extern long ZAC_POZIC;

/*
  Maska, kter odfiltruje z prvku pole adresy strutkury typu f_blok_adres_t adresu.
  (Zamaskuje ppadnou 1 na zatku.)
 */
#define BL_MASKA ((long)(1<<(sizeof(long)*8-1)))

#include <stdio.h>

/*
  Porovn pozice a zjist, jestli jsou shodn.

  parametr a prvn porovnvan pozice
  parametr b druh porovnvan pozice

  vrac 1, jsou-li pozice stejn, jinak 0.
 */
int stejne_pozice(TPozice *a, TPozice *b);

/*
  Vrac znakov popis figury k selnmu.

  parametr fig seln popis figury.
  vrac znak, kter figuru popisuje, mal psmeno v ppad ern figury,
        velk psmeno v ppad bl figury.
 */
char zn_fig(int fig);
/*
  Vype obsah pozice peten ze souboru.

  parametr file soubor, do nj bude pozice vypsna
  parametr pozice vypisovan pozice
  vrac 0, nedojde-li k chyb, jinak kd chyby dan konstantou v obk_ch.h
 */
int dump_f_pozice(FILE *file, f_pozice_t *pozice);

/*
  Vype pozici do souboru.

  parametr file soubor, do nj bude pozice vypsna.
  parametr pozice vypisovan pozice
  vrac 0, nedojde-li k chyb, jinak kd chyby dan konstantou v obk_ch.h
 */
int print_pozice(FILE *file, TPozice *pozice);

#endif
